#include "phy/signal.h"
#include "phy/channel.h"
#include "utils/dump.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdint.h>
#include <complex.h>
#include <fftw3.h>

void ofdm(complex *in_time, int size, complex *out_freq)
{
  complex *fft_in;
  complex *fft_out;
  fftw_plan p;

  fft_in = fftw_malloc(sizeof(complex) * size);
  fft_out = fftw_malloc(sizeof(complex) * size);
  if (fft_in == NULL || fft_out == NULL) abort();

  p = fftw_plan_dft_1d(size, fft_in, fft_out, FFTW_FORWARD, FFTW_ESTIMATE);

  memcpy(fft_in, in_time, sizeof(complex) * size);
  fftw_execute(p);
  memcpy(out_freq, fft_out, sizeof(complex) * size);

  fftw_destroy_plan(p);
  fftw_free(fft_in);
  fftw_free(fft_out);
}

void usage(void)
{
  printf("gimme <input file> <offset (byte)>\n");
  exit(0);
}

unsigned char in[T_S * 4];
complex in_time[T_U];
complex in_freq[T_U];

complex z_ref[T_U];
complex z_ref_time[T_S];
complex z_ref_freq[T_S];

complex cur[T_U];
complex prev[T_U];
complex dqpsk[T_U];

complex channel[T_U];

int main(int n, char **v)
{
  int i;
  int j;
  char *input_file = NULL;
  long offset = 0;
  FILE *input;
  //FILE *output;
  char output_file[128];

  for (i = 1; i < n; i++) {
    if (input_file == NULL) { input_file = v[i]; continue; }
    if (offset == 0) { offset = atol(v[i]); continue; }
    usage();
  }
  if (input_file == NULL || offset == 0) usage();

  input = fopen(input_file, "r");
  if (input == NULL) { perror(input_file); return 1; }
  fseek(input, offset, SEEK_SET);
  printf("#seek point set to %ld\n", ftell(input));

  /* compute sync sequence */
  compute_phase_ref(z_ref);
  do_signal(z_ref_time, z_ref);
  ofdm(&z_ref_time[Delta], T_U, z_ref_freq);
  text_dump("z_ref.txt", z_ref_freq, T_U);

  /* first symbol (sync) */
  if (fread(in, 1, T_S * 4, input) != T_S * 4) { printf("%s: bad file\n", input_file); return 1; }
  for (j = 0; j < T_U; j++) {
    int16_t re;
    int16_t im;
    re = in[(j+Delta)*4] | (in[(j+Delta)*4+1]<<8);
    im = in[(j+Delta)*4+2] | (in[(j+Delta)*4+3]<<8);
    in_time[j] = re/32768. + im/32768. * I;
  }
  ofdm(in_time, T_U, in_freq);

  /* channel estimate */
  for (i = 0; i < T_U; i++) {
    /* in = h * z_ref
     * in * z_ref* = h * z_ref * z_ref*
     * in * z_ref* = h * (a^2 + b^2)
     * in * z_ref* / cabs(z_ref)^2 = h
     */
    double abs2 = creal(z_ref_freq[i]) * creal(z_ref_freq[i]) + cimag(z_ref_freq[i]) * cimag(z_ref_freq[i]);
    if (abs2 < 1e-10)
      channel[i] = 0;
    else {
      channel[i] = in_freq[i] * conj(z_ref_freq[i]) / abs2;
      channel[i] = 1 / channel[i];
    }
  }
  text_dump("channel.txt", channel, T_U);

  channel_compensate(in_freq, channel, cur);
  text_dump("sync.txt", in_freq, T_U);
  text_dump("sync_compensated.txt", cur, T_U);

  for (i = 0; i < 3; i++) {
    if (fread(in, 1, T_S * 4, input) != T_S * 4) { printf("%s: bad file\n", input_file); return 1; }
    for (j = 0; j < T_U; j++) {
      int16_t re;
      int16_t im;
      re = in[(j+Delta)*4] | (in[(j+Delta)*4+1]<<8);
      im = in[(j+Delta)*4+2] | (in[(j+Delta)*4+3]<<8);
      in_time[j] = re/32768. + im/32768. * I;
    }
    ofdm(in_time, T_U, in_freq);
    memcpy(prev, cur, T_U * sizeof(complex));
    //memcpy(cur, in_freq, T_U*sizeof(complex));
    channel_compensate(in_freq, channel, cur);
    sprintf(output_file, "symbol%d_pre.txt", i);
    text_dump(output_file, in_freq, T_U);
    sprintf(output_file, "symbol%d_compensated.txt", i);
    text_dump(output_file, cur, T_U);

    for (j = 0; j < T_U; j++) {
      dqpsk[j] = cur[j] * conj(prev[j]);
    }
    sprintf(output_file, "symbol%d_dqpsk.raw", i);
    short_dump(output_file, dqpsk, T_U, 100);
    sprintf(output_file, "symbol%d_dqpsk.txt", i);
    text_dump(output_file, dqpsk, T_U);
#if 0
    output = fopen(output_file, "w");
    if (output == NULL) { perror(output_file); return 1; }
    for (j = 0; j < T_U; j++) {
      int16_t re;
      int16_t im;
      re = creal(dqpsk[j]) * 100;
      im = cimag(dqpsk[j]) * 100;
if (i == 3)
printf("%g %g\n", creal(dqpsk[j]), cimag(dqpsk[j]));
//if (isnan(creal(in_freq[j]))) abort();
      if (fwrite(&re, 1, 2, output) != 2) { printf("%s: error write\n", output_file); return 1; }
      if (fwrite(&im, 1, 2, output) != 2) { printf("%s: error write\n", output_file); return 1; }
//if (i == 0) printf("%d %d\n", re, im);
    }
    fclose(output);
#endif
  }

  fclose(input);
}
