#include "phy/signal.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdint.h>
#include <complex.h>
#include <fftw3.h>
#include <math.h>

void correlate(complex *z_ref_time, complex *in_time, int size, complex *corr)
{
  complex *z_ref;
  complex *in;
  complex *fft_in;
  complex *fft_out;
  fftw_plan p;
  fftw_plan p2;
  int i;

  fft_in = fftw_malloc(sizeof(complex) * size);
  fft_out = fftw_malloc(sizeof(complex) * size);
  z_ref = fftw_malloc(sizeof(complex) * size);
  in = fftw_malloc(sizeof(complex) * size);
  if (fft_in == NULL || fft_out == NULL || z_ref == NULL || in == NULL) abort();

  p = fftw_plan_dft_1d(size, fft_in, fft_out, FFTW_FORWARD, FFTW_ESTIMATE);
  p2 = fftw_plan_dft_1d(size, fft_in, fft_out, FFTW_BACKWARD, FFTW_ESTIMATE);

  memcpy(fft_in, z_ref_time, sizeof(complex) * size);
  fftw_execute(p);
  memcpy(z_ref, fft_out, sizeof(complex) * size);
  memcpy(fft_in, in_time, sizeof(complex) * size);
  fftw_execute(p);
  memcpy(in, fft_out, sizeof(complex) * size);

#if 0
  fft_in[0] = in[0] * z_ref[0] / size;
  for (i = 1; i < size; i++)
    fft_in[i] = in[i] * z_ref[size-i] / size;
  fftw_execute(p2);
  memcpy(corr, fft_out, sizeof(complex) * size);
#endif

#if 1
  for (i = 0; i < size; i++)
    fft_in[i] = in[i] * conj(z_ref[i]) / size;
  fftw_execute(p2);
  memcpy(corr, fft_out, sizeof(complex) * size);
#endif

  fftw_destroy_plan(p);
  fftw_destroy_plan(p2);
  fftw_free(fft_in);
  fftw_free(fft_out);
  fftw_free(z_ref);
  fftw_free(in);
}

void usage(void)
{
  printf("gimme <input file>\n");
  exit(0);
}

complex z_ref[K+1];
complex z_ref_time[T_F];
char in[T_F * 4];
complex in_time[T_F];
complex corr[T_F];

int main(int n, char **v)
{
  int i;
  char *input_file = NULL;
  FILE *input;

  for (i = 1; i < n; i++) {
    if (input_file == NULL) { input_file = v[i]; continue; }
    usage();
  }
  if (input_file == NULL) usage();

  input = fopen(input_file, "r");
  if (input == NULL) { perror(input_file); return 1; }
//fseek(input, 48000*4*8, SEEK_SET);
  if (fread(in, 1, T_F * 4, input) != T_F * 4) { printf("%s: bad file\n", input_file); return 1; }
  fclose(input);

  for (i = 0; i < T_F; i++) {
    int16_t re;
    int16_t im;
    re = (unsigned char)in[i*4] | ((unsigned char)in[i*4+1]*256);
    im = (unsigned char)in[i*4+2] | ((unsigned char)in[i*4+3]*256);
    in_time[i] = re/32768. + im/32768. * I;
#if 0
re = (signed char)in[i*2];
im = (signed char)in[i*2+1];
in_time[i] = re/128. + im/128. * I;
#endif
  }

  memset(z_ref_time, 0, sizeof(z_ref_time));

  compute_phase_ref(z_ref);
  do_signal(z_ref_time, z_ref);

#if 0
{
  FILE *f = fopen("z_ref.raw", "w");
  int16_t re = 0, im = 0;
  for (i = 0; i < 5999; i++) {
    fwrite(&re, 2, 1, f);
    fwrite(&im, 2, 1, f);
  }
  for (i = 0; i < T_F; i++) {
    int16_t re = creal(z_ref_time[i]) * 100;
    int16_t im = cimag(z_ref_time[i]) * 100;
    fwrite(&re, 2, 1, f);
    fwrite(&im, 2, 1, f);
  }
  fclose(f);
}
#endif

  correlate(z_ref_time, in_time, T_F, corr);

  double avg = 0;
  double max = 0;
  int max_pos = -1;
  for (i = 0; i < T_F; i++) {
    double v = cabs(corr[i]);
    avg += v;
    if (v > max) { max = v; max_pos = i; }
#if 1
    int16_t re = creal(corr[i]); // * 1000;
    int16_t im = cimag(corr[i]); // * 1000;
    fwrite(&re, 2, 1, stdout);
    fwrite(&im, 2, 1, stdout);
#endif
  }
  avg /= T_F;
  //printf("avg %g max %g max pos %d\n", avg, max, max_pos);
fprintf(stderr, "avg %g max %g max pos %d\n", avg, max, max_pos);
fprintf(stderr, "carg(max_pos) = %g %g\n", carg(corr[max_pos]), carg(corr[max_pos]) / (2 * M_PI) * 1000);
}
