#include "phy/signal.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdint.h>
#include <complex.h>
#include <fftw3.h>
#include <math.h>

void ofdm(complex *in_time, int size, complex *out_freq)
{
  complex *fft_in;
  complex *fft_out;
  fftw_plan p;

  fft_in = fftw_malloc(sizeof(complex) * size);
  fft_out = fftw_malloc(sizeof(complex) * size);
  if (fft_in == NULL || fft_out == NULL) abort();

  p = fftw_plan_dft_1d(size, fft_in, fft_out, FFTW_FORWARD, FFTW_ESTIMATE);

  memcpy(fft_in, in_time, sizeof(complex) * size);
  fftw_execute(p);
  memcpy(out_freq, fft_out, sizeof(complex) * size);

  fftw_destroy_plan(p);
  fftw_free(fft_in);
  fftw_free(fft_out);
}

void usage(void)
{
  printf("gimme <input file> <offset (byte)>\n");
  exit(0);
}

unsigned char in[T_S * 4];
complex in_time[T_S];
complex in_freq[T_S];

int main(int n, char **v)
{
  int i;
  int j;
  char *input_file = NULL;
  long offset = 0;
  FILE *input;
  FILE *output;
  char output_file[128];

  for (i = 1; i < n; i++) {
    if (input_file == NULL) { input_file = v[i]; continue; }
    if (offset == 0) { offset = atol(v[i]); continue; }
    usage();
  }
  if (input_file == NULL || offset == 0) usage();

  input = fopen(input_file, "r");
  if (input == NULL) { perror(input_file); return 1; }
  fseek(input, offset, SEEK_SET);
  printf("#seek point set to %ld\n", ftell(input));

  for (i = 0; i < 4; i++) {
    if (fread(in, 1, T_S * 4, input) != T_S * 4) { printf("%s: bad file\n", input_file); return 1; }
    for (j = 0; j < T_S; j++) {
      int16_t re;
      int16_t im;
      re = in[j*4] | (in[j*4+1]<<8);
      im = in[j*4+2] | (in[j*4+3]<<8);
      in_time[j] = re/32768. + im/32768. * I;
//printf("%d %g\n", j, cimag(in_time[j]));
    }
#if 0
memset(in_freq, 0, sizeof(in_freq));
compute_phase_ref(in_freq);
complex qq[T_S];
memset(qq, 0, sizeof(qq));
do_signal(qq, in_freq);
for (j = 0; j < T_U; j++) {
  in_time[j] = qq[j+Delta] / 32768;
if (isnan(creal(qq[j+Delta])) || isnan(cimag(qq[j+Delta]))) { printf("j %d j+delta %d\n", j, j+Delta); abort(); }
}
#endif
#if 1
{
  /* get fine frequency offset */
  complex freq_corr = 0;
  int i;
  for (i = T_U; i < T_S; i++)
    freq_corr += in_time[i] * conj(in_time[i - T_U]);
  fprintf(stderr, "carg(freq_corr)/ M_PI * 1000 = %g\n", carg(freq_corr) / M_PI * 1000);
}
#endif
    ofdm(&in_time[Delta], T_U, in_freq);
    sprintf(output_file, "symbol%d.raw", i);
    output = fopen(output_file, "w");
    if (output == NULL) { perror(output_file); return 1; }
    for (j = 0; j < T_U; j++) {
      int16_t re;
      int16_t im;
      re = creal(in_freq[j]) * 1000;
      im = cimag(in_freq[j]) * 1000;
//if (isnan(creal(in_freq[j]))) abort();
      if (fwrite(&re, 1, 2, output) != 2) { printf("%s: error write\n", output_file); return 1; }
      if (fwrite(&im, 1, 2, output) != 2) { printf("%s: error write\n", output_file); return 1; }
if (i == 0) printf("%d %d\n", re, im);
    }
    fclose(output);
  }

  fclose(input);
}
