#include <stdio.h>
#include <stdlib.h>
#include <string.h>

void usage(void)
{
  printf("gimme <message to decode (binary)>, K, poly 0, ..., poly n\n");
  exit(0);
}

int compute_poly(int poly, int shift, int k)
{
  int v = poly & shift;
  int r = 0;
  while (k) {
    r ^= v;
    v >>= 1;
    k--;
  }
  return r & 1;
}

int distance(int *expected, int *actual, int n)
{
  int ret = 0;
  int i;

  for (i = 0; i < n; i++)
    ret += expected[i] != actual[i];

  return ret;
}

void decode(char *m, int k, int *poly, int n_poly)
{
  int shift = 0;
  int i;
  int s;
  int n_bits = strlen(m) / n_poly;
  int n_states = 1 << (k-1);
  int error[n_bits+1][n_states];
  int prev[n_bits+1][n_states];
  int decoded[n_bits];
  int expected[n_poly];
  int actual[n_poly];
  int p;
  int next;
  int next_distance;

  if (n_bits * n_poly != strlen(m)) { printf("bad input\n"); exit(0); }

  memset(error, -1, sizeof(error));

  error[0][0] = 0;

  for (i = 0; i < n_bits; i++) {
    for (p = 0; p < n_poly; p++) {
      int c = *m; if (c == 0) abort();
      m++;
      if (c == '0') c = 0; else if (c == '1') c = 1; else { printf("bad message\n"); exit(0); }
      actual[p] = c;
    }

    for (s = 0; s < n_states; s++) {
      if (error[i][s] == -1) continue;

      /* input bit 0 */
      shift = s;
      for (p = 0; p < n_poly; p++)
        expected[p] = compute_poly(poly[p], shift, k);
      next = shift >> 1;
      next_distance = error[i][s] + distance(expected, actual, n_poly);
      if (error[i+1][next] == -1
         || next_distance < error[i+1][next]) {
        error[i+1][next] = next_distance;
        prev[i+1][next] = s;
      }

      /* input bit 1 (not for last k-1 bits, which are 0) */
      if (i >= n_bits - (k - 1)) continue;

      shift = s | (1 << (k -1));
      for (p = 0; p < n_poly; p++)
        expected[p] = compute_poly(poly[p], shift, k);
      next = shift >> 1;
      next_distance = error[i][s] + distance(expected, actual, n_poly);
      if (error[i+1][next] == -1
         || next_distance < error[i+1][next]) {
        error[i+1][next] = next_distance;
        prev[i+1][next] = s;
      }
    }
  }

  s = 0;
  for (i = n_bits; i > 0; i--) {
    decoded[i-1] = s >> (k - 2);
    s = prev[i][s];
  }

  printf("last error %d\n", error[n_bits][0]);

  for (i = 0; i < n_bits; i++) printf("%d", decoded[i]);
  printf("\n");
}

int main(int n, char **v)
{
  char *m = NULL;
  int k = -1;
  int poly[n];
  int n_poly = 0;
  int i;

  for (i = 1; i < n; i++) {
    if (m == NULL) { m = v[i]; continue; }
    if (k == -1) { k = atoi(v[i]); continue; }
    poly[n_poly] = atoi(v[i]);
    n_poly++;
  }

  if (k == -1 || n_poly == 0) usage();

  decode(m, k, poly, n_poly);

  return 0;
}
